unit GeoEllipsoids;

{$mode objfpc}{$H+}

interface

uses
  Classes, Contnrs, SysUtils;

type

 TEllipsoid = class
 private
   FEllipName : string;
   FMajorAxis : double;
   FInvFlat : double;
 public
   property EllipsoidName : string read FEllipName write FEllipName;
   property MajorAxis : double read FMajorAxis write FMajorAxis;
   property InvFlattening : double read FInvFlat write FInvFlat;
   constructor Create ;
   destructor  Destroy ; override ;
  end;

  TEllipsoidList = class (TStringList)
    private

    protected
      function GetEllipsoid(idx : Integer ) : TEllipsoid ;
    public
      constructor Create ;
      destructor  Destroy ; override ;
      procedure   AddEx (const Name : string; const MajorAxisA : double; const InvFlat : double);
      function    FindEx(const Name : string): TEllipsoid ;
  end;


implementation

//TEllipsoid
constructor TEllipsoid.Create;
begin
  inherited create;
end;

destructor TEllipsoid.Destroy;
begin
  inherited Destroy;
end;

//TEllipsoidList - Collection of TEllipsoid.
constructor TEllipsoidList.Create ;
begin
  inherited Create;
  Sorted := True ;

  AddEx('WGS84', 6378137, 298.257223563);
  AddEx('GRS80', 6378137, 298.257222101);
  AddEx('WGS72', 6378135, 298.260);
  AddEx('Australian 1965', 6378160, 298.250);
  AddEx('Krasovsky 1940', 6378245, 298.3);
  AddEx('International (1924)', 6378388, 297);
  AddEx('Clake 1880', 6378249.1, 293.460);
  AddEx('Clarke 1866', 6378206.4, 294.980);
  AddEx('Airy 1830', 6377563.4, 299.320);
  AddEx('Bessel 1841', 6377397.2, 299.150);
  AddEx('Everest 1830', 6377276.345, 300.8017);
  AddEx('Hayford 1909', 6378388.0, 296.999362081575);
  AddEx('North American 1927', 6378206.4, 294.978698213898);
  AddEx('NAD 83', 6378137, 298.257223563);
end ;


destructor TEllipsoidList.Destroy ;
var
  i : Integer ;
begin
 for i:= Count -1 downto 0 do
 begin
   if Assigned( Objects[i] ) then
      TEllipsoidList( Objects[i] ).Free ;
  end ;
  inherited Destroy;
end ;

procedure TEllipsoidList.AddEx(const Name          : string ;
                               const MajorAxisA     : double ;
                               const InvFlat : double);
var
  e : TEllipsoid ;
  pos : Integer ;
begin
  if Find(Name, pos ) then begin
    if Assigned(Objects[pos]) then
      TEllipsoid(Objects[pos]).Free ;
  end ;
  pos := Add(Name) ;

  e := TEllipsoid.Create ;
  with e do
  begin
    EllipsoidName := Name;
    MajorAxis     := MajorAxisA;
    InvFlattening := InvFlat;
  end ;
  Objects[pos] := e;

end;

function TEllipsoidList.FindEx(const Name : String): TEllipsoid ;
var
  pos : Integer ;
begin
  if Find( Name, pos ) then
    Result := TEllipsoid(Objects[pos])
  else
    Result := nil ;
end ;

function TEllipsoidList.GetEllipsoid(idx : Integer) : TEllipsoid ;
begin
  Result := TEllipsoid( Objects[ idx ] ) ; ;
end ;

end.

