unit Unit1; 

{$mode objfpc}{$H+}

interface

uses
  Classes, SysUtils, FileUtil, LResources, Forms, Controls, Graphics, Dialogs,
  StdCtrls, ExtCtrls, GeoEllipsoids, GeoCompute;

type

  { TForm1 }

  TForm1 = class(TForm)
    cmdEx1: TButton;
    cmdEx2: TButton;
    cmdCompute: TButton;
    cmdClose: TButton;
    cmbEllipsoids: TComboBox;
    cmbHem: TComboBox;
    Label7: TLabel;
    txtZoneNo: TEdit;
    Label3: TLabel;
    Label4: TLabel;
    Label5: TLabel;
    Label6: TLabel;
    txtE: TEdit;
    txtLat: TEdit;
    Label1: TLabel;
    Label2: TLabel;
    radComputeType: TRadioGroup;
    txtLong: TEdit;
    txtN: TEdit;
    procedure cmdCloseClick(Sender: TObject);
    procedure cmdComputeClick(Sender: TObject);
    procedure cmdEx1Click(Sender: TObject);
    procedure cmdEx2Click(Sender: TObject);
    procedure FormCreate(Sender: TObject);
    procedure FormDestroy(Sender: TObject);
    procedure radComputeTypeClick(Sender: TObject);
  private
    { private declarations }
    FEllipses : TEllipsoidList;
  public
    { public declarations }
  end; 

var
  Form1: TForm1; 
  function Coordinate(const X, Y : double) : TCoordinate;
implementation

function Coordinate(const X, Y : double) : TCoordinate;
begin
  result.X := X;
  result.Y := Y;
end;

{ TForm1 }

procedure TForm1.FormCreate(Sender: TObject);
var
  i : integer;
begin
  FEllipses := TEllipsoidList.Create;
  for i := 0 to FEllipses.Count - 1 do
  begin
    cmbEllipsoids.Items.Add(TEllipsoid(FEllipses.Objects[i]).EllipsoidName);
  end;
  cmbEllipsoids.ItemIndex := FEllipses.Count - 1; //Default with WGS84
  //start with UTM to Geographic computation.
  txtN.Enabled := true;
  txtE.Enabled := true;
  txtZoneNo.Enabled := true;
  cmbHem.Enabled := true;

  txtLat.Readonly := true;
  txtLong.Readonly := true;
end;

procedure TForm1.cmdCloseClick(Sender: TObject);
begin
  Close;
end;

procedure TForm1.cmdComputeClick(Sender: TObject);
var
  utm2geo : TUTM2Geo;
  geo2utm : TGeo2UTM;
  ell : TEllipsoid;
  x, y : double;
  utmproj : TUTMProjection;
  geocoor, utmcoor : TCoordinate;
begin
  ell := FEllipses.FindEx(cmbEllipsoids.Text);

  if (radComputeType.ItemIndex = 0) then //UTM to Geographic
  begin
    x := strtofloat(txtE.Text);
    y := strtofloat(txtN.Text);
    utmcoor := Coordinate(x, y);
    utmproj.ZoneNo := strtoint(txtZoneNo.Text);
    if (cmbHem.ItemIndex = 0)then
      utmproj.LatHem := hemNorth
    else
      utmproj.LatHem := hemSouth;

    try
      utm2geo := TUTM2Geo.Create;
      utm2geo.Ellipsoid := ell;
      utm2geo.UTMProjection := utmproj;
      utm2Geo.UTMCoordinate := utmcoor;
      utm2Geo.Compute;
      txtLat.Text := format('%11.8f', [utm2geo.GeoCoordinate.Y]);
      txtLong.Text := format('%11.8f', [utm2geo.GeoCoordinate.X]);
    finally
      utm2geo.Free;
    end;
  end
  else  //Geographic to UTM
  begin
    x := strtofloat(txtLong.Text);
    y := strtofloat(txtLat.Text);
    geocoor := Coordinate(x, y);

    try
      geo2utm := TGeo2UTM.Create;
      geo2utm.Ellipsoid := ell;
      geo2utm.GeoCoordinate := geocoor;
      geo2utm.Compute;
      utmproj := geo2utm.UTMProjection;
      txtN.Text := format('%11.3f', [geo2utm.UTMCoordinate.Y]);
      txtE.Text := format('%11.3f', [geo2utm.UTMCoordinate.X]);
      txtZoneNo.Text := format('%3d', [utmproj.ZoneNo]);
      if (utmproj.LatHem = hemNorth) then
        cmbHem.ItemIndex := 0
      else
        cmbHem.ItemIndex := 1;
    finally
      geo2utm.Free;
    end;
  end
end;

procedure TForm1.cmdEx1Click(Sender: TObject);
begin
  radComputeType.ItemIndex := 0; //UTM to Geographic
  txtN.Text := '3885802.712';
  txtE.Text := '468327.864';
  txtZoneNo.Text := '11'; //UTM Zone = 11
  cmbHem.ItemIndex := 0; //North

  txtLat.Text := '';
  txtLong.Text := '';
end;

procedure TForm1.cmdEx2Click(Sender: TObject);
begin
  radComputeType.ItemIndex := 1; //Geographic to UTM
  txtLat.Text := '-37.6528211388889';
  txtLong.Text := '143.926495527778';

  txtN.Text := '';
  txtE.Text := '';
  txtZoneNo.Text := '';
end;

procedure TForm1.FormDestroy(Sender: TObject);
begin
  FEllipses.Free;
end;

procedure TForm1.radComputeTypeClick(Sender: TObject);
begin
  if (radComputeType.ItemIndex = 0) then // UTM to Geographic
  begin
    cmdCompute.Caption := '<==';
    txtN.Enabled := true;
    txtE.Enabled := true;
    txtZoneNo.Enabled := true;
    cmbHem.Enabled := true;
    txtN.SetFocus;
    txtLat.Readonly := true;
    txtLong.Readonly := true;
  end
  else
  begin
    cmdCompute.Caption := '==>';   //Geographic to UTM
    txtN.Readonly := true;
    txtE.Readonly := true;
    txtZoneNo.Readonly := true;
    cmbHem.Readonly := true;
    txtLat.Enabled := true;
    txtLong.Enabled := true;
    txtLat.SetFocus;
  end;
end;

initialization
  {$I unit1.lrs}

end.

